#import <Foundation/Foundation.h>

static unsigned ObserverContext;
@interface Observer : NSObject
{
    NSObject *_target;
}
@property(readonly) NSObject *target;
- initWithTarget:(id)target;
- (void)stopObservation;
@end
@implementation Observer
@synthesize target = _target;
- initWithTarget:(id)target;
{
    _target = [target retain];
    [_target addObserver:self forKeyPath:@"value" options:NSKeyValueObservingOptionNew|NSKeyValueObservingOptionOld context:&ObserverContext];
    return self;
}
- (void)stopObservation;
{
    [self.target removeObserver:self forKeyPath:@"value"];
}
- (void)observeValueForKeyPath:(NSString *)keyPath ofObject:(id)object change:(NSDictionary *)change context:(void *)context;
{
    NSLog(@"%@.%@ context:%p change:%@", object, keyPath, context, change);
}
@end

static unsigned SubObserverContext;
@interface SubObserver : Observer
@end
@implementation SubObserver
- initWithTarget:(id)target;
{
    if (!(self = [super initWithTarget:target]))
        return nil;
    [target addObserver:self forKeyPath:@"value" options:NSKeyValueObservingOptionNew|NSKeyValueObservingOptionOld context:&SubObserverContext];
    return self;
}
@end

@interface Target : NSObject
{
    NSString *value;
}
@property(copy) NSString *value;
@end
@implementation Target
@synthesize value;
@end

int main (int argc, const char * argv[])
{
    NSAutoreleasePool * pool = [[NSAutoreleasePool alloc] init];

    Target *target = [[Target alloc] init];
    SubObserver *observer = [[SubObserver alloc] initWithTarget:target];
    
    NSLog(@"setting value with two observations:");
    target.value = @"foo";
    
    // Which observation of 'value' does this remove??  -removeObserver:forKeyPath: doesn't let you specify the context, so this is undefined.
    // NOTE: it does not remove both; it does only remove one.
    NSLog(@"setting value after calling -removeObserver:forKeyPath: once");
    [observer stopObservation];
    target.value = @"bar";

    NSLog(@"setting value after calling -removeObserver:forKeyPath: twice");
    [observer stopObservation];
    target.value = @"blegga";

    [pool drain];
    return 0;
}
